package MCS::CGI::Gaestebuch;

# $Id: Gaestebuch.pm,v 1.58 2007/08/30 09:54:38 sebi Exp $
# $Log: Gaestebuch.pm,v $
# Revision 1.58  2007/08/30 09:54:38  sebi
# * fixed Schreibfehler Fehlermeldung deutlicher gemacht.
#
# Revision 1.57  2007/06/20 15:55:05  nico
# [UPD] paar meldungen weniger nach STDERR
#
# Revision 1.56  2007/06/20 15:50:10  nico
# [UPD] CTS#73208 PATH_INFO gif,jpg,png werden ignoriert
#
# Revision 1.55  2006/11/22 13:42:23  nico
# [err] print vergessen
#
# Revision 1.54  2006/11/22 13:06:37  nico
# [UPD] Falle Konfigdatei nicht exisiteiert, wird skript abgebrochen
#

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $debug);

$debug = 0;
use MCS::CGI::Conf;
use MCS::CGI::Standard;
use MCS::CGI::HTML::Template;
use CGI;

use POSIX;
use Time::Local;

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader
	 MCS::CGI::Standard
);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	 gaestebuch_admin
	 gaestebuch_form
	 gaestebuch_extended

);

@EXPORT_OK = qw(

);

$VERSION = sprintf "%d.%02d", ( q$Revision: 1.58 $ =~ /\d+/g );

=head1 NAME

MCS::CGI::Gaestebuch - Standard-Gaestebuch

=head1 VERSION

     $Id: Gaestebuch.pm,v 1.58 2007/08/30 09:54:38 sebi Exp $


=head1 SYNOPSIS


Hier ist das gesamte CGI-Skript (fE<uuml>nf Zeilen):

  #! /usr/bin/perl -wT
  use strict;
  $ENV{PATH} = '/bin:/usr/bin'; # Fuer Taint-Checking
  use MCS::CGI::Gaestebuch qw( gaestebuch_extended );
  gaestebuch_extended();


=head1 DESCRIPTION

MCS::CGI::Gaestebuch Standard-Gaestebuch (Administration).

=head1 SYNOPSIS


Hier ist das gesamte CGI-Skript (fE<uuml>nf Zeilen) fE<uuml>r die Administration:

  #! /usr/bin/perl -wT
  use strict;
  $ENV{PATH} = '/bin:/usr/bin'; # Fuer Taint-Checking
  use MCS::CGI::Gaestebuch qw( gaestebuch_admin );
  gaestebuch_admin();


=head1 DESCRIPTION

MCS::CGI::Gaestebuch Administrationsschnittstelle.


=cut

# Preloaded methods go here.

sub gaestebuch_form {
	my ( $cgi, $cfg ) = @_;
	unless ( $ENV{PATH_TRANSLATED} ) {
		$ENV{PATH_TRANSLATED}
			 = "$ENV{DOCUMENT_ROOT}/gaestebuch/gaestebuch.conf";
	}
	unless ($cgi) { $cgi = new CGI; }
	unless ($cfg) { $cfg = new MCS::CGI::Conf; }

	my $parse = new MCS::CGI::HTML::Template;
	my $template = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'form' );

	$parse->var( DATUM =>
			  format_date_string( $cfg->val( 'gb_text', 'date_only_format' ) ) );

	if ( $cgi->param('antwort_zu_name') && $cgi->param('antwort_zu_beitrag') )
	{
		$parse->var( IS_ANTWORT => sub { return 1; } );
		$parse->var( ANTWORT_ZU_BEITRAG => $cgi->param('antwort_zu_beitrag') );
		$parse->var( ANTWORT_ZU_NAME    => $cgi->param('antwort_zu_name') );

	} else {
		$parse->var( IS_ANTWORT => sub { return 0; } );
	}

	print $cgi->header();
	$parse->parse_file(
						 $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'form' ) );

}

sub gaestebuch_extended {

	my ( $cgi, $cfg ) = @_;
	unless ($cgi) { $cgi = new CGI; }

	if ( $ENV{PATH_TRANSLATED} ) {
		if (    ( $ENV{PATH_INFO} =~ /.gif$/ )
			  || ( $ENV{PATH_INFO} =~ /.jpg$/ )
			  || ( $ENV{PATH_INFO} =~ /.png$/ ) )
		{
			$cgi->header( -status => "503 Out of Resources",
							  -type   => "text/plain" );

			print "$ENV{PATH_INFO} hier nicht gueltig\n";
			exit();
		}
	} else {
		$ENV{PATH_TRANSLATED}
			 = "$ENV{DOCUMENT_ROOT}/gaestebuch/gaestebuch.conf";
	}
	unless ( -f $ENV{PATH_TRANSLATED} ) {
		print $cgi->header();
		print "Konfigurationsdatei Datei erwartet";
		exit();
	}

	unless ($cfg) { $cfg = new MCS::CGI::Conf; }

	if ( $cgi->param('action') && ( $cgi->param('action') =~ /newentry/ ) ) {

		# neuer Eintrag
		# set umask, damit Dateien world-readable sind (text and ggf. Pic
		umask(0022);

		if ( store_new_entry( $cfg, $cgi ) ) {
			if ( $cfg->val( 'gb_mail', 'mail' ) ) {
				send_entry( $cfg, $cgi );
			}
			rebuild_gaestebuch_html( $cfg, $cgi );
		}
	}

	else {
		listEntries( $cfg, $cgi );
	}

}

=pod

=head2 Feldvariablen

=over

=item * realname

=item * website

=item * email

=item * comment

=item * remark0 .. remark9

=back

=head2 Funktionen

=over

=item B<store_new_entry> (cfg, $cgi)

speichert den neuen Eintrag nach ueberpruefung.

=cut

sub store_new_entry {
	my ( $cfg, $cgi ) = @_;

	my $status = $cfg->val( 'gb_default', 'status' ) || "released";
	$status =~ s/ //g;
	my $user_realname = $cgi->param('realname')
		 or exit_error( $cfg, $cfg->val( "gb_text", "needed_name" ) );
	my $user_website = $cgi->param('website');
	if ($user_website) {
		unless ( $user_website =~ /https{0,1}:\/\//i ) {
			exit_error( $cfg,
							$cfg->val( "gb_text", "needed_http_in_webadress" ) );
		}
	}
	my $user_email;
	$user_email = $cgi->param('email') || "";
	if ( $cfg->val( "gb_text", "needed_email" ) ) {
		$cgi->param('email')
			 or exit_error( $cfg, $cfg->val( "gb_text", "needed_email" ) );
	}
	if ( $cgi->param('email') ) {
		unless ( MCS::CGI::Standard::is_valid_email( $cgi->param('email') ) ) {
			exit_error( $cfg,
							$cfg->val( "gb_text", "wrong_email" )
								 || "E-Mail Adresse ist ung&uuml;ltig"
			);
		}
	}
	my $user_comment = $cgi->param('comment')
		 or exit_error( $cfg, $cfg->val( "gb_text", "needed_comment" ) );

	my ( $i, @remarks );
	for ( $i = 0; $i < 10; $i++ ) {
		if ( $cgi->param("remark${i}") ) {
			$remarks[$i] = $cgi->param("remark${i}");
		} else {
			print STDERR "Remark$i nicht angegeben\n" if ($debug);
			print STDERR "\$cfg->val(\"gb_text\", \"needed_remark$i\")"
				 if ($debug);
			print STDERR $cfg->val( "gb_text", "needed_remark$i" );
			if ( $cfg->val( "gb_text", "needed_remark$i" ) ) {
				print STDERR "\n\neigentlich muesste das Srkitp sterben\n";
				exit_error( $cfg, $cfg->val( "gb_text", "needed_remark$i" ) );
			}
		}
	}

	unless ( $cfg->val( 'gb_entry', 'allow_html' ) ) {
		$user_realname = disallow_html_set_br($user_realname);
		$user_website  = disallow_html_set_br($user_website);
		$user_email    = disallow_html_set_br($user_email);
		$user_comment  = disallow_html_set_br($user_comment);
		for ( $i = 0; $i < 10; $i++ ) {
			if ( $remarks[$i] ) {
				$remarks[$i] = disallow_html_set_br( $remarks[$i] );
			}
		}
	}

	if (    $cfg->val( 'gb_entry', 'max_word_length' )
		  || $cfg->val( 'gb_entry', 'max_rows' ) )
	{
		foreach ( $user_realname, $user_comment ) {
			unless ( is_valid_length_of_field(
											  $cfg->val( 'gb_entry', 'max_word_length' ),
											  $cfg->val( 'gb_entry', 'max_rows' ),
											  $_
						)
				 )
			{
				exit_error( $cfg,
								$cfg->val( "gb_text", "entry_to_long" )
									 || "Beitrag enthaelt mehr als 50 Zeilen oder ein Wort mit mehr als 60 Zeichen"
				);
			}
		}
		for ( $i = 0; $i < 10; $i++ ) {
			if ( $remarks[$i] ) {
				unless ( is_valid_length_of_field(
											  $cfg->val( 'gb_entry', 'max_word_length' ),
											  $cfg->val( 'gb_entry', 'max_rows' ),
											  $remarks[$i]
							)
					 )
				{
					exit_error( $cfg,
									$cfg->val( "gb_text", "entry_to_long" )
										 || "Beitrag enthaelt mehr als 50 Zeilen oder ein Wort mit mehr als 60 Zeichen"
					);
				}
			}
		}
	}

	my $template = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'entry' );
	my $parse = new MCS::CGI::HTML::Template;

	my ($picfilename);

	if ( $cfg->val( 'gb_files', 'badwords' ) ) {

		if ( badwords_apear( $cfg,
									$ENV{DOCUMENT_ROOT}
										 . $cfg->val( 'gb_files', 'badwords' ),
									[ $user_comment, $user_realname,
									  $user_website, $user_email
									]
			  )
			 )
		{
			$status = "onhold";
		}
	}

	if ( $cfg->val( 'forbidden', 'names' ) ) {
		my @bad_names = split /,/, $cfg->val( 'forbidden', 'names' );
		if (@bad_names) {
			foreach (@bad_names) {
				$status = "onhold" if ( $_ && $user_realname =~ /$_/i );
			}
		}
	}

	# Name fuer Gaestebucheintrag (Datei 'YYYY-MM-DD-HHMMSS-PID') generieren.
	my $file_name = create_filename();

	if ( $cgi->param('pic_upload') ) {
		my ( $format, $up_name, $pictype );
		$up_name = $cgi->param('pic_upload');
		$pictype = $cgi->uploadInfo($up_name)->{'Content-Type'};

		$format
			 = ( $pictype eq "image/gif" ) ? "gif"
			 : ( ( $pictype eq "image/jpg" ) ? "jpg"
				  : (   ( $pictype eq "image/jpeg" ) ? "jpg"
						: ( ( $pictype eq "image/pjpeg" ) ? "jpg" : undef )
				  )
			 );

		if ($format) {

			#store_picture
			$picfilename = store_picture_file( $cfg, $cgi, $file_name, $format );
			$parse->var( IMAGE => sub { return $picfilename; } );
			$parse->var( IMG_SRC => $picfilename );
		} else {
			exit_error( $cfg,
							$cfg->val( "gb_text", "invalid_picture_format" ) . ": "
								 . $pictype
			);
		}
	} else {
		$parse->var( IMAGE => sub { return 0; } );
	}

	for ( my $image_count = 1; $image_count <= 100; $image_count++ ) {
		if ( $cgi->param("pic_upload${image_count}") ) {
			my ( $format, $up_name, $pictype );
			$up_name = $cgi->param("pic_upload${image_count}");
			$pictype = $cgi->uploadInfo($up_name)->{'Content-Type'};

			$format
				 = ( $pictype eq "image/gif" ) ? "gif"
				 : ( ( $pictype eq "image/jpg" ) ? "jpg"
					  : (   ( $pictype eq "image/jpeg" ) ? "jpg"
							: ( ( $pictype eq "image/pjpeg" ) ? "jpg" : undef )
					  )
				 );

			if ($format) {

				#store_picture
				my $picfilename;
				$picfilename =
					 store_picture_file( $cfg, $cgi,
												$file_name . "-" . $image_count,
												$format, "pic_upload$image_count" );
				$parse->var( "IMAGE$image_count" => sub { return $picfilename; }
				);
				$parse->var( "IMG_SRC$image_count" => $picfilename );
			} else {
				exit_error( $cfg,
								$cfg->val( "gb_text", "invalid_picture_format" ) );
			}
		} else {
			$parse->var( "IMAGE$image_count" => sub { return 0; } );
			print STDERR "IMAGE$image_count solle nicht errscheinen\n"
				 if ($debug);
		}
	}

	$parse->var( REALNAME         => $user_realname );
	$parse->var( ESCAPED_REALNAME => $cgi->escape($user_realname) );
	$parse->var( WEBSITE_LINK     => $user_website );
	$parse->var( WEBSITE          => sub { return $user_website; } );
	$parse->var( COMMENT          => $user_comment );
	if ( $cgi->param('bgcolor') ) {
		$parse->var( BGCOLOR => get_bgcolor( $cfg, $cgi ) || "" );
	} else {
		$parse->var( BGCOLOR => "" );
	}

	if ($user_email) {
		$parse->var( EMAIL_DEFINED => sub { return 1; } );
		$parse->var( EMAIL => $user_email );
	} else {
		$parse->var( EMAIL_DEFINED => sub { return 0; } );
	}
	$parse->var(
		TIME => format_date_string( $cfg->val( 'gb_text', 'time_only_format' ) )
	);
	$parse->var(
			  DATE => format_date_string( $cfg->val( 'gb_text', 'date_format' ) )
					|| "" );
	$parse->var( DATUM =>
				  format_date_string( $cfg->val( 'gb_text', 'date_only_format' ) )
				  || "" );

	for ( $i = 0; $i < 10; $i++ ) {
		if ( $remarks[$i] ) {
			$parse->var( "REMARK${i}_DEFINED" => sub { return 1; } );
		} else {
			$parse->var( "REMARK${i}_DEFINED" => sub { return 0; } );
		}
		$parse->var( "REMARK$i" => $remarks[$i] || "" );
	}

	$parse->var( FILENAME => $file_name );
	if ( $cgi->param('antwort_zu_beitrag') && $cgi->param('antwort_zu_name') )
	{
		$parse->var( IS_ANTWORT => sub { return 1; } );
		$parse->var( ANTWORT_ZU_BEITRAG => $cgi->param('antwort_zu_beitrag') );
		$parse->var( ANTWORT_ZU_NAME    => $cgi->param('antwort_zu_name') );

	} else {
		$parse->var( IS_ANTWORT => sub { return 0; } );
	}
	$file_name
		 = $ENV{DOCUMENT_ROOT}
		 . $cfg->val( 'gb_path', $status ) . "/"
		 . $file_name;
	if ( $file_name =~ /^([-_\w.\/]+)$/ ) {
		$file_name = $1;
		open( OUT, ">$file_name" )
			 || fatal_exit_error(
							$cfg,
							"Kann Datei $file_name nicht zum schreiben oeffnen: $! Evtl fehlen Rechte fr das Gaestebuchverzeichnis?",
							"kann Datei nicht zum Schreiben oeffen."
			 );

		my $old_fh = select(OUT);
		$parse->parse_file(
						$ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'entry' ) );
		select($old_fh);
		close(OUT);
		display_thanks( $cfg, $cgi, $file_name, $status );
	} else {
		fatal_exit_error(
							$cfg,
							"Kann Datei $file_name nicht zum schreiben oeffnen: $! Evtl fehlen Rechte fr das Gaestebuchverzeichnis?",
							"kann Datei nicht zum Schreiben oeffen."
		);
	}
	return 1;

}

=item get_bgcolor

 get_bgcolor($cfg, $cgi)

gibt die Hintergrundfarbe zurueck, die in $cgi->param(bgcolor) definiert war,
wenn diese in $cfg->val('','') enthalten ist, sonst die erste Farbe  aus

=cut

sub get_bgcolor {
	my ( $cfg, $cgi ) = @_;
	my ( @bgcolors, $bgcolor );
	@bgcolors = split /\s*,\s*/, $cfg->val( 'gb_template_flag', 'bgcolor' );
	if ( $cgi->param('bgcolor') =~ /(\#[\daAbBcCdDeEfF]{6})/ ) {
		$bgcolor = $1;
		foreach (@bgcolors) {
			if ( $_ =~ /$bgcolor/i ) {
				return $_;
			}
		}
	}
	return $bgcolors[0];

}

=item display_thanks ( $cfg, $cgi, $file_name, $status);

Gibt eine Dankes-Seite (BestE<auml>tigung aus)

=cut

sub display_thanks {
	my ( $cfg, $cgi, $file_name, $status ) = @_;
	my $parse = new MCS::CGI::HTML::Template;
	my $entry;

	open( ENT, "$file_name" )
		 or fatal_exit_error( $cfg,
									 "kann $file_name nicht oeffnen",
									 "Kann Eintrag nicht zum lesen &ouml;ffenen" );
	while (<ENT>) {
		$entry .= $_;
	}
	close ENT;

	$parse->var( ENTRY => $entry );
	$parse->var( STATUS_MSG => $cfg->val( 'gb_status_msg', $status ) || "" );
	print $cgi->header();
	$parse->parse_file(
					  $ENV{DOCUMENT_ROOT} . $cfg->val( "gb_template", "thanks" ) );
}

=item B<format_date_string>($format, $time)

gibt einen Datumsstring der das aktuelle oder das optional
uebergeben Datum $ime formatiert zurueck.

=cut

sub format_date_string {
	my ( $format, $time ) = @_;
	if ($time) {
		return POSIX::strftime( $format, localtime($time) );
	} else {
		return POSIX::strftime( $format, localtime(time) );
	}
}

=item B<store_picture_file> ($cfg, $cgi, $filename, $format)

speichert bild


=cut

sub store_picture_file {
	my ( $cfg, $cgi, $filename, $format, $cgi_pic_upload ) = @_;
	$cgi_pic_upload ||= "pic_upload";
	my $infilename = $cgi->param($cgi_pic_upload);

	my $picfilename = $ENV{DOCUMENT_ROOT};
	$picfilename .= $cfg->val( 'gb_picture', 'pic_files' );
	$picfilename .= "/$filename";
	my $width  = $cfg->val( 'gb_picture', 'max_width' )  || 100;
	my $height = $cfg->val( 'gb_picture', 'max_height' ) || 100;

	my ( $bytesread, $buffer );

	# write (binray) file to right place
	if ( $picfilename =~ /^([-_\w.\/]+)$/ ) {
		$picfilename = "$1.$format";
		open( OUTFILE, ">$picfilename" );
		while ( $bytesread = read( $infilename, $buffer, 1024 ) ) {
			print OUTFILE $buffer;
		}
		close(OUTFILE);

		my $prog_mogrify = $cfg->val( "programm", "mogrify" );
		die
			 "$prog_mogrify insecure (must be an absolute path without whitespace)\n"
			 unless $prog_mogrify =~ m|^/[^\s]+$|;
		die "$prog_mogrify not found or not executable.\n"
			 unless -x $prog_mogrify;
		system(
			"$prog_mogrify -format $format -geometry \"${width}x${height}>\" $picfilename"
		);
		$picfilename =~ s/$ENV{DOCUMENT_ROOT}//;
		return $picfilename;
	} else {
		return undef;
	}

}

=item B<badwords_apear> ($cfg, $file_wordlist, $strings_to_check_ref)

prueft, ob ein Stopwort in einem der

=cut

sub badwords_apear {

	my ( $cfg, $file_wordlist, $strings_to_check_ref ) = @_;

	my @BAD;
	my $word;
	my $words;
	my @strings_to_check = @$strings_to_check_ref;
	if ( -f $file_wordlist ) {
		unless ( open BADWORDS, "<$file_wordlist" ) {
			print STDERR "$0 Kann $file_wordlist nicht oeffnen $!\n";
			fatal_exit_error( $cfg,
									"$0 Kann $file_wordlist nicht oeffnen $!\n",
									"Kann eine notwendige Datei nicht &ouml;ffnen"
			);
		}
		($words) = <BADWORDS>;
		close BADWORDS;

		@BAD = split /,/, $words;
		chomp @BAD;
		foreach (@BAD) {
			$word = $_;
			if ( $word =~ /^$/ ) {

				#ignore empty String
				next;
			}
			foreach (@strings_to_check) {
				if ( $_ =~ /$word/i ) {
					return 1;
				}
			}
		}
		return 0;
	} else {
		print STDERR "BADWORDLIST definiert, aber Datei existiert nicht\n";
		send_mail( $cfg,
					  "Check Badwordlist",
					  "$0: Die Datei $file_wordlist existiert nicht ",
					  $ENV{SERVER_ADMIN} );
	}
	return 0;
}

=item B<create_filename>

gibt einen Dateinamen in form von YYYY-MM-DD-HHMMSS-PID zurueck.


=cut

sub create_filename {

	# dateinamen generieren aus datum
	my ( $sek, $min, $std, $Mtag, $Monat, $Jahr, $wtag, $jtag, $isdst )
		 = localtime(time);
	$Monat++;
	if ( $std <= 9 )  { $std  = "0" . $std; }
	if ( $min <= 9 )  { $min  = "0" . $min; }
	if ( $sek <= 9 )  { $sek  = "0" . $sek; }
	if ( $Mtag <= 9 ) { $Mtag = "0" . $Mtag; }
	$Jahr += 1900;
	if ( $Monat <= 9 ) { $Monat = "0" . $Monat; }
	my $datei_name
		 = $Jahr . "-"
		 . $Monat . "-"
		 . $Mtag . "-"
		 . $std
		 . $min
		 . $sek . "-$$";

	# $$ gibt aktuelle Prozess-ID wieder
	# new markiert Eintrag als "neu"

	return ($datei_name);

}

=item B<listEntries>

gibt das Gaestebuch aus.

=cut

sub listEntries {
	my ( $cfg, $cgi, $flag_ref ) = @_;
	my ( $parse, $template, $prevText, $nextText );
	$template = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'template' );
	$parse = new MCS::CGI::HTML::Template;

	# Fange bei 0 an, wenn $next < 0;
	my $next
		 = ( defined( $cgi->param('next') ) && $cgi->param('next') > 0 )
		 ? $cgi->param('next')
		 : 0;
	my $max_per_page = $cfg->val( 'gb_entry', 'show_max' )    || 10;
	my $max_entries  = $cfg->val( 'gb_entry', 'max_entries' ) || 100;
	print STDERR "gb_path: $$flag_ref{gb_path} \n" if ($debug);
	my $entrydir
		 = $$flag_ref{gb_path}
		 ? $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', $$flag_ref{'gb_path'} )
		 : get_entrydir( $cfg, $cgi );
	print STDERR "entrydir: $entrydir\n" if ($debug);
	my $script_name = $ENV{REQUEST_URI};
	$script_name =~ s/(.*?)\?.*/$1/;

	$script_name =~ s/gaestebuch-admin.pl/gaestebuch.pl/;

	unless ( $$flag_ref{skip_header} ) {
		print $cgi->header( -type => 'text/html', -expires => '-1d' );
	}
	opendir DIR, $entrydir
		 or fatal_exit_error( $cfg,
									 "Kann $entrydir nicht oeffnen ! $!",
									 "kann Verzeichnis nicht zum Lesen oeffen." );
	my @entries;
	if ( $cfg->val( 'gb_entry', 'nur_tag' ) ) {
		my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
			 = localtime( time + 86400 * $cgi->param('nur_tag') );
		$mon++;
		$year += 1900;
		if ( $mon < 10 )  { $mon  = "0$mon" }
		if ( $mday < 10 ) { $mday = "0$mday" }
		@entries = reverse sort grep { /^$year-$mon-$mday/; } readdir(DIR);
		$parse->var( DATUM =>
							  format_date_string(
							 $cfg->val( 'gb_text', 'date_only_format' ),
							 ( time + 86400 * $cgi->param('nur_tag') )
							  )
		);

	} else {
		@entries = reverse sort grep { /^\d/; } readdir(DIR);
	}
	closedir DIR;
	foreach (@entries) {
		print STDERR $_, "\n" if ($debug);
	}
	my $entriesLeng = @entries;

	#-----------Loesche alle ueber max_entries---------------------->

	my $lastEntry = $#entries;

	if ( $entriesLeng > $max_entries ) {
		my $file2delete = "$entrydir/$entries[$lastEntry]";
		$file2delete =~ s/^.*($ENV{DOCUMENT_ROOT}.*)/$1/;
		$file2delete = $1;
		if ( -e $file2delete ) {
			print STDERR " Datei : $file2delete wird jetzt geloescht\n"
				 if ($debug);
			system("rm $1") == 0 or return 0;
		}
	}

	#-------->

	my $limit = $next + $max_per_page;
	if ( $limit > $entriesLeng ) { $limit = $entriesLeng; }

	my $i     = 0;
	my @stuff = ();

	my $entry;
	for ( $i = $next; $i < $limit; $i++ ) {
		open( ENT, "$entrydir/$entries[$i]" )
			 or fatal_exit_error( $cfg,
										 "kann $entrydir/$entries[$i] nicht oeffnen",
										 "Kann Eintrag nicht zum lesen &ouml;ffenen"
			 );
		print STDERR "$entrydir/$entries[$i]\n" if ($debug);
		$entry = "";
		while (<ENT>) {
			$entry .= $_;
		}
		push @stuff, { ENTRY => $entry };

		close ENT;
	}

	# "zurueck" und "weiter" Links erzeugen.
	# --------------------->

	my $prevEntries = $next - $max_per_page;
	my $nextEntries = $next + $max_per_page;

	my $back    = $max_per_page;
	my $forward = $max_per_page;
	$forward = $entriesLeng - $limit
		 if ( ( $entriesLeng - $limit ) < $max_per_page );

	$prevText = $cfg->val( 'gb_text', 'back_text' );
	$prevText =~ s/\@\@back\@\@/$back/;
	$nextText = $cfg->val( 'gb_text', 'forward_text' );
	$nextText =~ s/\@\@forward\@\@/$forward/;

	my $PREVLINK = "\&nbsp;";
	my $NEXTLINK = "\&nbsp;";

	$PREVLINK = "<a href=\"$script_name?next=$prevEntries\">$prevText</a>"
		 if ( $next > 0 );
	$NEXTLINK = "<a href=\"$script_name?next=$nextEntries\">$nextText</a>"
		 if ( $next < $entriesLeng - $max_per_page );

	# Template oeffnen und zum Browser schicken
	my $von = $next + 1;
	$parse->var( PREVLINK => $PREVLINK );
	$parse->var( ALL      => $entriesLeng );
	$parse->var( START    => $von );
	$parse->var( END      => $limit );
	$parse->var( NEXTLINK => $NEXTLINK );
	$parse->var( ENTRIES  => \@stuff );

	open FILE, $template
		 or fatal_exit_error( $cfg,
									 "Cannot open $template:$!",
									 "Template kann nicht geoeffnet werden" );
	$parse->parse_file( \*FILE );
	close FILE;

}

=item B<rebuild_gaestebuch_html>

schreibt ggf. Startseite fuers Gaestebuch.

=cut

sub rebuild_gaestebuch_html {
	my ( $cfg, $cgi ) = @_;
	my ( @file_stat, $html_file, $html_file_new );
	unless ( $cfg->val( 'gb_path', 'start_page' ) ) {
		return;
	}
	$html_file = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', 'start_page' );
	if ( $html_file =~ /^([0-9-_\w.\/]+)$/ ) {
		$html_file     = $1;
		$html_file_new = $html_file . "." . $$;
		open( OUT, ">$html_file_new" )
			 || fatal_exit_error(
					  $cfg,
					  "Kann Datei $html_file_new nicht zum schreiben oeffnen ! $!",
					  "kann Datei nicht zum Schreiben oeffen."
			 );

		flock( OUT, 2 );    #LOCK_EX: 2;
		my $old_fh = select(OUT);
		$cgi->param( -name  => 'next',
						 -value => '0' );
		listEntries( $cfg, $cgi, { skip_header => 1, gb_path => 'released' } );
		select($old_fh);
		print STDERR __LINE__ . " ListEntries kam zurueck\n" if ($debug);
		flock( OUT, 8 );    #LOCK_UN: 8;
		close(OUT);

		# testen of $html_file_new > 0
		@file_stat = stat($html_file_new);
		if ( $file_stat[7] > 0 ) {
			rename( $html_file_new, $html_file );
		} else {
			fatal_exit_error( $cfg,
									"Kann neue Datei $html_file_new war leer! $!",
									"kann Datei nicht zum Schreiben oeffen." );
		}
	} else {
		fatal_exit_error(
							$cfg,
							"Kann Datei $html_file nicht zum schreiben oeffnen ! $!",
							"kann Datei nicht zum Schreiben oeffen."
		);
	}
}

=item B<get_entrydir>


=cut

sub get_entrydir {
	my ( $cfg, $cgi ) = @_;
	unless ( is_valid_admin( $cfg, $cgi ) ) {
		return $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', 'released' );
	} else {

		if ( $cgi->param('admin_view') ) {
			return $ENV{DOCUMENT_ROOT}
				 . $cfg->val( 'gb_path', $cgi->param('admin_view') );
		} else {
			my $status = $cfg->val( 'gb_default', 'status' );
			$status =~ s/\s$//g;

#	    	print STDERR "ENTRYDIR: $ENV{DOCUMENT_ROOT}" .  $cfg->val('gb_path', $status). "#${status}#\n"; ;
			return $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', $status );
		}
	}
}

=item B<fatal_exit_error> ($cfg, $error_msg, $public_error_msg)

verschickt eine Fehlermeldung an den Admin und schreibt Fehlermeldung ins Errorlogfile


=cut

sub fatal_exit_error {
	my ( $cfg, $error_msg, $public_error_msg, $public_long_error_msg ) = @_;
	my $parse = new MCS::CGI::HTML::Template;
	my $tmpl = $ENV{DOCUMENT_ROOT} . $cfg->val( "gb_template", "error" );

	print STDERR "$ENV{SERVER_NAME}\t$error_msg\n";
	print "Content-type: text/html\n\n";
	$parse->var( ERROR => $public_error_msg );
	$parse->var( ERROR_LANG_TEXT => $public_long_error_msg || "" );
	$parse->parse_file($tmpl);
	send_mail( $cfg, "Fatal error $0", $error_msg, $ENV{SERVER_ADMIN} );
	exit(0);
}

=item B<exit_error> ($cfg, hinweis)

schreibt Fehlermeldung an Benutzer, keine Fehlermeldung an Admin etc.

=cut

sub exit_error {
	my ( $cfg, $public_error_msg, $public_long_error_msg ) = @_;
	my $parse = new MCS::CGI::HTML::Template;
	my $tmpl = $ENV{DOCUMENT_ROOT} . $cfg->val( "gb_template", "error" );
	print "Content-type: text/html\n\n";
	$parse->var( ERROR => $public_error_msg );
	$parse->var( ERROR_LANG_TEXT => $public_long_error_msg || "" );
	$parse->parse_file($tmpl);
	exit(0);
}

sub exit_message {
	my ( $cfg, $cgi, $header, $hint1 ) = @_;
	my $parse = new MCS::CGI::HTML::Template;
	my $tmpl
		 = $ENV{DOCUMENT_ROOT} . $cfg->val( "gb_template", "admin_message" );
	my $script_name = $ENV{REQUEST_URI};
	$script_name =~ s/(.*?)\?.*/$1/;

	my $query_string = "admin_name=" . $cgi->param('admin_name');
	$query_string .= "&admin_pass=" . $cgi->param('admin_pass');
	$query_string .= "&admin_view=" . $cgi->param('admin_view');
	$query_string .= "&next=" . ( $cgi->param('next') - 1 );

	print "Content-type: text/html\n\n";
	$parse->var( SCRIPT_NAME  => $script_name );
	$parse->var( QUERY_STRING => $query_string );
	$parse->var( HINT_HEADER  => $header );
	$parse->var( HINT1        => $hint1 || "" );
	$parse->parse_file($tmpl);
	exit(0);
}

=item B<send_mail> ($cfg, $subject, $msg, $to, $cc)

versende Mail

=cut

sub send_mail {
	my ( $cfg, $subject, $msg, $to, $cc ) = @_;
	$cc ||= "";

	# Check the sendmail command for security
	my $sendmail = $cfg->val( 'mail', 'sendmail' );
	die "$sendmail insecure (must be an absolute path without whitespace)\n"
		 unless $sendmail =~ m|^/[^\s]+$|;
	die "$sendmail not found or not executable.\n"
		 unless -x $sendmail;
	open MAIL, "| $sendmail -t -oi"
		 or die "Cannot open sendmail: $!\n";

	print MAIL <<EOF;
To: $to
Cc: $cc
Subject: $subject

$msg

EOF

	close(MAIL);

}

=item B<is_valid_length_of_field> ($max_wordlength, $max_rows, $eingabe)

prueft ob diese Bedingung eingehalten wird.
prec: $max_wordlength ||= 60;
      $max_rows       ||= 50;
      $eingabe        zupruefender Text
post: undef  oder 1

=cut

sub is_valid_length_of_field {
	my ( $max_wordlength, $max_rows, $eingabe ) = @_;
	$max_wordlength ||= 60;
	$max_rows       ||= 50;
	my ( @rows, $line, @words, );
	@rows = split /\n/, $eingabe;

	if ( $#rows > $max_rows ) {
		print STDERR "Beitrag hat zu viele Zeilen  ($#rows > $max_rows)\n"
			 if ($debug);
		return undef;
	}
	foreach (@rows) {
		$line = shift @rows;
		@words = split /\s/, $line;

		print STDERR "@words\n" if ($debug);

		foreach (@words) {
			if ( length($_) > $max_wordlength ) {
				print STDERR
					 "Beitrag hat zu lange Worte (length($_) > $max_wordlength)\n"
					 if ($debug);
				return undef;
			}
		}
	}

	#wenn Programm hier ankommt stimmt die Zeilenzahl und Wortlaenge,
	#also return true
	return 1;
}

=item B<is_valid_admin> ($cfg,$cgi)

prueft ob sich ein gueltiger Administrator einloggt.
Rueckgabe wert ist WAHR oder FALSCH.

=cut

sub is_valid_admin {
	my ( $cfg, $cgi ) = @_;
	if (    defined( $cgi->param('admin_name') )
		  && defined( $cgi->param('admin_pass') ) )
	{

		return (   ( $cfg->val( 'admin', 'name' ) eq $cgi->param('admin_name') )
				  && ( $cfg->val( 'admin', 'pass' ) eq $cgi->param('admin_pass') )
		);
	} else {
		return 0;
	}
}

=item B<gaestebuch_admin> ()

Hauptfunktion der Admin-Schnittstelle

=cut

sub gaestebuch_admin {

	unless ( $ENV{PATH_TRANSLATED} ) {
		$ENV{PATH_TRANSLATED}
			 = "$ENV{DOCUMENT_ROOT}/gaestebuch/gaestebuch.conf";
	}

	my $cgi = new CGI;
	my $cfg = new MCS::CGI::Conf;

	unless ( is_valid_admin( $cfg, $cgi ) ) {
		exit_error( $cfg, $cfg->val( "gb_text", "admin_only" ) );
	}

	# nun wissen wir es ist ein berechtigter Admin

	if ( $cgi->param('admin_action') ) {
		if ( $cgi->param('admin_action') =~ /move2/ ) {
			if ( $cgi->param('admin_action') =~ /move2onhold/ ) {

				# schiebe Beitrag nach $cfg->val("gb_path", "onhold") ;
				move_beitrag( $cfg, $cgi,
								  (         $cgi->param('admin_view')
											|| $cfg->val( 'gb_default', 'status' )
								  ),
								  "onhold"
				);
			} elsif ( $cgi->param('admin_action') =~ /move2released/ ) {

				# schiebe Beitrag nach $cfg->val("gb_path", "released") ;
				move_beitrag( $cfg, $cgi,
								  (         $cgi->param('admin_view')
											|| $cfg->val( 'gb_default', 'status' )
								  ),
								  "released"
				);
			} elsif ( $cgi->param('admin_action') =~ /move2refused/ ) {

				# schiebe Beitrag nach $cfg->val("gb_path", "refused") ;
				move_beitrag( $cfg, $cgi,
								  (         $cgi->param('admin_view')
											|| $cfg->val( 'gb_default', 'status' )
								  ),
								  "refused"
				);
			}
		} elsif ( $cgi->param('admin_action') eq "delete" ) {

			#loesche Beitrag
			remove_beitrag( $cfg, $cgi,
								 (         $cgi->param('admin_view')
										  || $cfg->val( 'gb_default', 'status' )
								 )
			);
		} elsif ( $cgi->param('admin_action') eq "rebuildhtml" ) {

			#HTML-Starte seine neue bauehn
			rebuild_gaestebuch_html( $cfg, $cgi );
			exit_message( $cfg,
							  $cgi,
							  $cfg->val( "gb_text", "admin_htmlrebuild" )
									|| "Startseite wurde neu produiziert:",
							  $cfg->val( "gb_path", "start_page" )
			);

		} elsif ( $cgi->param('admin_action') eq "edit" ) {
			exit_error( $cfg,
							"<b>edit Beitrage  noch nicht implementiert</b> " );
		} else {

			# diese Stelle darf eigentlich nicht erreicht werden!
			exit_error( $cfg, "Ungueltige Aktion ausgewaehlt" );
		}
	} else {

		# admin_action ist nicht uebermittelt:
		admin_listEntries( $cfg, $cgi );
	}
}

=item B<admin_listEntries>

gibt das Gaestebuch mit Optionen fuer Administratoren aus.

=cut

sub admin_listEntries {
	my ( $cfg, $cgi ) = @_;
	my ( $parse, $template, $prevText, $nextText );
	$template
		 = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_template', 'admin_template' );
	$parse = new MCS::CGI::HTML::Template;

	# Fange bei 0 an, wenn $next < 0;
	my $next = ( $cgi->param('next') > 0 ) ? $cgi->param('next') : 0;
	my $max_per_page = $cfg->val( 'gb_entry', 'show_max' )    || 10;
	my $max_entries  = $cfg->val( 'gb_entry', 'max_entries' ) || 100;
	my $entrydir     = get_entrydir( $cfg,    $cgi );
	my $script_name  = $ENV{REQUEST_URI};
	$script_name =~ s/(.*?)\?.*/$1/;

	print $cgi->header( -type => 'text/html', -expires => '-1d' );

	opendir DIR, $entrydir
		 or fatal_exit_error( $cfg,
									 "Kann $entrydir nicht oeffnen ! $!",
									 "kann Verzeichnis nicht zum Lesen oeffen." );
	my @entries = reverse sort grep { /^\d/; } readdir(DIR);
	closedir DIR;

	my $entriesLeng = @entries;

	my $limit = $next + $max_per_page;
	if ( $limit > $entriesLeng ) { $limit = $entriesLeng; }

	my $i     = 0;
	my @stuff = ();
	my $entry;
	for ( $i = $next; $i < $limit; $i++ ) {
		open( ENT, "$entrydir/$entries[$i]" )
			 or fatal_exit_error( $cfg,
										 "kann $entrydir/$entries[$i] nicht oeffnen",
										 "Kann Eintrag nicht zum lesen &ouml;ffenen"
			 );
		print STDERR "$entrydir/$entries[$i]\n" if ($debug);
		$entry
			 = "<input type=\"checkbox\" name=\"entry\" value=\"$entries[$i]\"> ";
		$entry .= $cfg->val( 'gb_text', 'admin_following_entry' );
		$entry .= "\n";
		while (<ENT>) {
			$entry .= $_;
		}
		push @stuff, { ENTRY => $entry };
		close ENT;
	}

	# "zurueck" und "weiter" Links erzeugen.
	# --------------------->

	my $prevEntries = $next - $max_per_page;
	my $nextEntries = $next + $max_per_page;

	my $back    = $max_per_page;
	my $forward = $max_per_page;
	$forward = $entriesLeng - $limit
		 if ( ( $entriesLeng - $limit ) < $max_per_page );

	$prevText = $cfg->val( 'gb_text', 'back_text' );
	$prevText =~ s/\@\@back\@\@/$back/;
	$nextText = $cfg->val( 'gb_text', 'forward_text' );
	$nextText =~ s/\@\@forward\@\@/$forward/;

	my $PREVLINK     = "\&nbsp;";
	my $NEXTLINK     = "\&nbsp;";
	my $query_string = $cgi->query_string();

	$query_string =~ s/next=\d+&*//g;
	$PREVLINK
		 = "<a href=\"$script_name?next=$prevEntries&$query_string\">$prevText</a>"
		 if ( $next > 0 );
	$NEXTLINK
		 = "<a href=\"$script_name?next=$nextEntries&$query_string\">$nextText</a>"
		 if ( $next < $entriesLeng - $max_per_page );

	# Template oeffnen und zum Browser schicken
	my $von = $next + 1;
	$parse->var( PREVLINK => $PREVLINK );

	$parse->var( ALL      => $entriesLeng );
	$parse->var( START    => $von );
	$parse->var( END      => $limit );
	$parse->var( NEXTLINK => $NEXTLINK );
	$parse->var( ENTRIES  => \@stuff );

	$parse->var( ACTUAL_VIEW => select_box_of_actual_view( $cfg, $cgi ) );
	$parse->var(ADMIN_ACTION => select_box_of_possible_actions( $cfg, $cgi ) );
	$parse->var( SCRIPT_NAME => $script_name || "fehlt" );
	$parse->var( ADMIN_NAME => $cgi->param('admin_name') );
	$parse->var( ADMIN_PASS => $cgi->param('admin_pass') );

	open FILE, $template
		 or fatal_exit_error( $cfg,
									 "Cannot open $template:$!",
									 "Template kann nicht geoeffnet werden" );
	$parse->parse_file( \*FILE );
	close FILE;

}

=item B<select_box_of_possible_actions> ($cfg,$cgi);

Rueckgabewert ist ein String mit einer SELECT-Box mit den moeglichen Aktionen.

=cut

sub select_box_of_possible_actions {
	my ( $cfg, $cgi ) = @_;
	my $r_string;

	$r_string = '<select name="admin_action">
<option value="">' . $cfg->val( 'gb_text', 'please_select' );
	$r_string .= '<option value="move2released" >'
		 . $cfg->val( 'gb_text', 'move2released' );
	$r_string .= '<option value="move2onhold" >'
		 . $cfg->val( 'gb_text', 'move2onhold' );

	$r_string .= '<option value="move2refused" >'
		 . $cfg->val( 'gb_text', 'move2refused' );
	$r_string .= '<option value="delete" >' . $cfg->val( 'gb_text', 'delete' );
	if (    $cfg->val( 'gb_text', 'rebuildhtml' )
		  && $cfg->val( 'gb_path', 'start_page' ) )
	{
		$r_string .= '<option value="rebuildhtml" >'
			 . $cfg->val( 'gb_text', 'rebuildhtml' );
	}
	$r_string .= '</select >';

		 return $r_string;

}

=item B<select_box_of_actual_view> ($cfg,$cgi)

Rueckgabewert ist ein String mit einer SELECT-Box

=cut

sub select_box_of_actual_view {
	my ( $cfg, $cgi ) = @_;
	my $r_string;

	my $default_status
		 = ( $cgi->param('admin_view') || $cfg->val( 'gb_default', 'status' ) );
	$default_status =~ s/\s//g;

	$r_string
		 = '<select name="admin_view" onChange="this.form.next.value=0;this.form.submit();">';
	$r_string
		 .= '<option value="'
		 . $default_status . '">'
		 . $cfg->val( 'gb_text', $default_status )
		 . "</option>";

	$r_string
		 .= '<option value="released" >' . $cfg->val( 'gb_text', 'released' );
	$r_string .= '<option value="onhold" >' . $cfg->val( 'gb_text', 'onhold' );
	$r_string
		 .= '<option value="refused" >' . $cfg->val( 'gb_text', 'refused' );
	$r_string .= '</select >';

		 return $r_string;
}

=item B<move_beitrag> ($cfg,$cgi, $former_status, $new_status)

verschieb Beitraege von einem Verzeichnis in ein anderes.


=cut

sub move_beitrag {
	my ( $cfg, $cgi, $former_status, $new_status ) = @_;

	my ( $old_dir, $new_dir, @beitraege, $beitrag, $beitrag_neu );
	if ( $former_status eq $new_status ) {
		exit_error( $cfg, $cfg->val( "gb_text", "same_status" ) );
	} else {
		unless ( $former_status && $new_status ) {
			exit_error(
				$cfg,
				"Nicht beide Stati definiert alt: $former_status new: $new_status"
			);
		}
		$old_dir = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', $former_status );
		$new_dir = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', $new_status );
		@beitraege = $cgi->param('entry');
		$old_dir =~ s/^([0-9-_\.\w\/]+)$/$1/;
		$old_dir = $1;

		#remove ".."
		$old_dir =~ s/\.\.//g;
		$new_dir =~ s/^([0-9-_\.\w\/]+)$/$1/;
		$new_dir = $1;
		$new_dir =~ s/\.\.//g;
		foreach (@beitraege) {
			if ( $_ =~ /^([-_\w.0-9\/]+)$/ ) {
				$beitrag     = $1;
				$beitrag_neu = $beitrag;
				$beitrag_neu =~ s/-new$//;
				unless ( rename( "$old_dir/$beitrag", "$new_dir/$beitrag_neu" ) )
				{
					fatal_exit_error(
									  $cfg,
									  "Entry cound't be moved <br>\n"
											. "$old_dir/$beitrag , $new_dir/$beitrag<br>"
											. "alt: $former_status new: $new_status",
									  "Entry cound't be moved <br>\n"
					);
				}
			} else {
				exit_error( $cfg, "invalid signature: $_ " );
			}
		}
		exit_message( $cfg, $cgi,
						  $cfg->val( "gb_text", "admin_moved" ),
						  $cfg->val( "gb_text", "$new_status" ) );

	}

}

=item B<send_entry> ($cfg, $cgi)

ruft send_mail() mit auf, um dem Administrator den neuen Eintrag zu zu senden.

=cut

sub send_entry {
	my ( $cfg, $cgi ) = @_;

	my ( $subject, $msg, $to, $cc );

	$subject = $cfg->val( 'gb_mail', 'subject' )
		 || "neuer Eintrag im Gaestebuch";
	$to = $cfg->val( 'gb_mail', 'to' ) || $ENV{SERVER_ADMIN};
	$cc = $cfg->val( 'gb_mail', 'cc' ) || "";

	$msg .= $cgi->param('realname') . " ";
	$msg .= $cgi->param('email') . "\n\n";
	$msg .= $cgi->param('website') . "\n" if ( $cgi->param('website') );
	$msg .= $cgi->param('comment') . "\n\n";

	send_mail( $cfg, $subject, $msg, $to, $cc );

}

=item B<disallow_html_set_br> ($string)

transformiert alle "<"-Zeichen in "&lt;" und alle ">"-Zeichen in "&gt;"
ausserdem ersetzt es "\n" in "<br>"

=cut

sub disallow_html_set_br {
	my ($string) = @_;
	$string =~ s/</&lt;/g;
	$string =~ s/>/&gt;/g;
	$string =~ s/
/\<br\>/g;
	$string =~ s/\n/\<br\>/g;
	return $string;
}

=item B<remove_beitrag> ($cfg,$cgi, $former_status )

loescht Beitraege unwiederbringlich.


=cut

sub remove_beitrag {
	my ( $cfg, $cgi, $former_status, ) = @_;

	my ( $old_dir, $new_dir, @beitraege, $beitrag, $error_nr );
	$old_dir = $ENV{DOCUMENT_ROOT} . $cfg->val( 'gb_path', $former_status );
	@beitraege = $cgi->param('entry');
	$old_dir =~ s/^([0-9-_\.\w\/]+)$/$1/;
	$old_dir = $1;

	#remove ".."
	$old_dir =~ s/\.\.//g;
	foreach (@beitraege) {
		if ( $_ =~ /^([-_\w.0-9\/]+)*$/ ) {
			$beitrag = $1;
			if ( -f "$old_dir/$beitrag" ) {
				unless ( unlink("$old_dir/$beitrag") ) {
					fatal_exit_error(
						  $cfg,
						  "Entry couldn't be removed <br>\n" . "$old_dir/$beitrag ",
						  "Entry cound't be removed <br>\n"
					);
				}
			} else {
				exit_message( $cfg,
								  "Entry couldn't be removed <br>\n",
								  "Entry cound't be removed <br>\n"
										. "file is missing $old_dir/$beitrag "
				);
			}

			#loesche assoziiertes Bild
			my $picfilename = $ENV{DOCUMENT_ROOT};
			$picfilename
				 .= $cfg->val( 'gb_picture', 'pic_files' ) . "/${beitrag}";

			if ( $picfilename =~ /^([-_\w.0-9\/]+)(-new)*$/ ) {
				$picfilename = $1;
			} else {
				$picfilename = "";
			}

			if (    -f "$picfilename.jpg"
				  || -f "$picfilename.png"
				  || -f "$picfilename.gif" )
			{
				print STDERR "Removing picfile: #${picfilename}*#\n" if ($debug);
				if ( $error_nr = system("rm ${picfilename}*") ) {
					fatal_exit_error(
						  $cfg,
						  "Picture for Entry cound't be removed $error_nr <br>\n"
								. "$picfilename ",
						  "Entry cound't be removed picture - Error: $error_nr<br>\n"
					);
				}
			} else {
				print STDERR "picfile removed: #${picfilename}*#\n" if ($debug);
			}
		} else {
			exit_error( $cfg, "invalid signature: $_ " );
		}
	}
	exit_message( $cfg, $cgi, $cfg->val( "gb_text", "admin_removed" ) );
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

# Below is the stub of documentation for your module. You better edit it!

=back

=head1 AUTHOR

Klaus Tillmann <nico@mcs-cityline.net> + Ludger Ester <lester@mcs-cityline.net>


=head1 SEE ALSO

perl(1).

=cut
