package MCS::CGI::SMS;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $debug);

$debug = 0;

##
## MCS::CGI::SMS -- Module for handling SMS-Gateways
##
## $Id: SMS.pm,v 1.3 2000/11/20 14:26:52 nico Exp $
## Copyright (C) 2000 MCS Moorbek Computer Systeme GmbH
## Author: Klaus Tillmann <nico@cityline.net>
##


use MCS::CGI::Conf;


use AnyDBM_File;
#use DB_File;
use SDBM_File;
use Fcntl;


require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = sprintf "%d.%02d", (q$Revision: 1.3 $ =~ /\d+/g);



# Preloaded methods go here.

# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

MCS::CGI::SMS - Handler for SMS Gateway

=head1 SYNOPSIS

  use MCS::CGI::SMS;


=head1 DESCRIPTION

MCS::CGI::SMS supports the cityline-wide use of the same interfaces
to the sms-gateway
 

=cut 

# Autoload methods go after =cut, and are processed by the autosplit program.





=head2 Funktionen

=over

=item I<new>

B<new> legt ein neues SMS Objekt an

=cut

sub new {

    my $class = shift;
    my $self = {}; # Anonymen Namens-Hash anlegen;

    bless ($self, $class);   # Segnen

    unless( $self->{'cfg'} = new MCS::CGI::Conf ){
	print STDERR "Initialisierung von SMS via MCS::CGI::Conf  schlug fehl\n";
	return wantarray ? () : undef;
    }

    return $self;
}


=item B<user_destination_allowed> ($user,$destination, \$error_code)

Prueft ob die Grenze der erlaubten Anzahl von SMS-Nachrichten nicht 
E<uuml>berschritten ist.


=cut


sub user_destination_allowed {
    my ($self, $user,$destination, $error_code_ref) = @_;
    
    my ($user_file, $user_dest_file);
    
    $user_file =  $self->{'cfg'}->val('sms_gateway','sms_lockfile_users') ;
    $user_file =  $ENV{DOCUMENT_ROOT} . $user_file;
    unless( $self->{'cfg'}->val('sms_gateway','sms_lockfile_users') ){
	print STDERR "Kann Wert fuer sms_lockfile nicht ermitteln!\n";
    }
    else {
	print __LINE__,"\n",  $self->{'cfg'}->val('sms_gateway','sms_lockfile_users') ,"\n" if($debug);
    }
    $user_dest_file = $self->{'cfg'}->val('sms_gateway','sms_lockfile_user_destinations');
    $user_dest_file =  $ENV{DOCUMENT_ROOT} . $user_dest_file;
    my %USER_HASH;
    
    if($self->test_max_timestamps_for_key_in_file($user_file, $user,
					    $self->{'cfg'}->val('sms_gateway','sms_rate_max_per_user'),
					   $self->{'cfg'}->val('sms_gateway','sms_rate_timeinterval') )){
	if($self->test_max_timestamps_for_key_in_file($user_dest_file, $user."-".$destination,
					       $self->{'cfg'}->val('sms_gateway','sms_rate_max_per_user__destination'),
					       $self->{'cfg'}->val('sms_gateway','sms_rate_timeinterval') )){
	    
	    return 1;
	}
	else{
	    $$error_code_ref = 371;
	    return 0;
	}
    }
    else{
	$$error_code_ref = 370;
	return 0;
    }

}
    
=item B<test_max_timestamps_for_key_in_file> ($filename, $key)

ist eine Funktion die nur intern aufgerufen werden sollte!!

Sie prE<uuml>ft, ob ein ein User bzw. User-Destination-Paar nicht mehr eintraege als erlaubt hat.

=cut
sub test_max_timestamps_for_key_in_file{
    my ($self, $filename, $key, $max_entries, $expire_time)  =@_;
    
    print ($filename,"\t",$key,"\t", $max_entries,"\t", $expire_time,"\n") if($debug);
    my (@arr, %USER_HASH, $return_val);

    open(LOCK, ">${filename}.lock") 
	|| die "kann Lockdatei ${filename}.lock nicht oeffnen \n";
	
    flock(LOCK,2); #LOCK_EX: 2; 

#    tie %USER_HASH, "AnyDBM_File", $filename, O_RDWR|O_CREAT,0660
    tie (%USER_HASH, 'SDBM_File', "${filename}.dbmx", O_RDWR|O_CREAT,0660 )


	|| die "kann DBM Datei $filename nicht oeffnen\n";
 
    @arr = split / +/,$USER_HASH{$key};
    print STDERR "ARR:" . scalar(@arr) ,"\t" if($debug);
    if(scalar(@arr) < $max_entries){
	#fuege neuen Zeitstempel hinzu
	$USER_HASH{$key} .= time ." ";
	$return_val = 1;
    }
    #pruefe ob erstes elment aelter als time - $expire_time ist.
    elsif($arr[0] < (time - $expire_time)) {
	while($arr[0] && ($arr[0] <  (time - $expire_time)) ) {
	    #hole erstes Element von links:
	    shift(@arr);
	}
	push(@arr, time . " ");
	$USER_HASH{$key} = join(" ", @arr);
	$return_val = 1;
    }
    else{
	print STDERR "zuviele Eintraege\n" if ($debug);
	$return_val = 0;
    }

    untie %USER_HASH;
    flock(LOCK,8); #LOCK_UN: 8; 
    close(LOCK);
    print STDERR "Ende von test\nreturn_val:$return_val\n" if ($debug);
    return $return_val;

}


=item I<forced_referer_ok> (\$error_code)

falls [sms_gateway]sms_forced_referer_host definiert ist, wird geprueft 
ob der E<uuml>bermittelte HTTP_REFERER tatsE<uuml>chlich mit dem in der 
Konfiguration angegeben E<uuml>bereinstimmt.

=cut

sub forced_referer_ok{
    my ($self, $error_code_ref) = @_;

    my $forced_referer_host = $self->{'cfg'}->val('sms_gateway','sms_forced_referer_host') ;
    
    if($forced_referer_host){
	unless($ENV{HTTP_REFERER}){
	    $$error_code_ref = 360;
	    return 0;
	}
	if( $ENV{HTTP_REFERER} =~ /$forced_referer_host/){
	    return 1;
	}
	else{
	    $$error_code_ref = 361;
	    return 0;
	}
    }
    return 1;
}


=item I<send_sms> ($user, $destination_nr, $message_header, $message_body, $message_footer)

sendet die SMS, wenn nicht der erzwungener Referer fehlt.

=cut

sub send_sms {
    my ($self,
	$user, 
	$destination_nr, 
	$message_header,
	$message_body,
	$message_footer,
	$error_code_ref) = @_; 

    my ($sms_subject);
    
    unless( $self->forced_referer_ok($error_code_ref)){
   	return wantarray ? () : undef; 
    }
    $sms_subject .= "uname=" . $self->{'cfg'}->val('sms_gateway','sms_user');
    $sms_subject .= "&pword=" . $self->{'cfg'}->val('sms_gateway','sms_password');
    
    # mindestens message_header or -footer muss mit der Definition ueberein
    # stimmen um Missbrauch zu unterbinden.
    my $default_message_header = $self->{'cfg'}->val('sms_gateway','sms_default_message_header');
    my $default_message_footer = $self->{'cfg'}->val('sms_gateway','sms_default_message_footer');
    if( $message_header =~ /$default_message_header/
	||
	$message_footer =~ /$default_message_footer/ ){
	unless( length($message_header) 
		+ length($message_body)
		+ length($message_footer) <= $self->{'cfg'}->val('sms_gateway','sms_message_length') ){
	    print STDERR "Message_lenght: " . length($message_header) . " " . length($message_body) . " "  . length($message_footer) . "\n" if($debug);
	    $$error_code_ref = "372";
	    return 0;
	}
	if($self->send_sms2gateway ($destination_nr .'@' . $self->{'cfg'}->val('sms_gateway','sms_mail_gateway'),
				    $sms_subject,
				    $message_header . $message_body . $message_footer, 
				    $error_code_ref)){
	    
	    return 1;
	}
    }
    else{
	$$error_code_ref = "301";
    }

    return wantarray ? () : undef;

}

=item I<send_sms2gateway>

Prec: Es muss vorher gepE<uuml>ft worden sein, dass das Senden dieser 
SMS gE<uuml>ltig ist. 
 
Post: SMS wurde an das MCS Gateway E<uuml>bermittelt!

=back 

=cut 


sub send_sms2gateway{
    my ($self,$to,$sms_subject,$sms_message) = @_;

    print STDERR "funktion send_sms2gateway reached\n" if($debug);
    # Check the sendmail command for security
    my $sendmail = $self->{'cfg'}->val('mail', 'sendmail');
    die
	"$sendmail insecure (must be an absolute path without whitespace)\n"
	    unless $sendmail =~ m|^/[^\s]+$|;
    die "$sendmail not found or not executable.\n"
	unless -x $sendmail;
    
    # Send  mail
    open MAIL, "| $sendmail -t -oi"
	or die "Cannot open sendmail: $!\n";

    print STDERR
	"To: $to\n",
	"From: $ENV{SERVER_ADMIN}\n",
	"Subject: $sms_subject \n\n",
	$sms_message  if($debug);
    
    print MAIL
	"To: $to\n",
	"From: $ENV{SERVER_ADMIN}\n",
	"Subject: $sms_subject \n\n",
	$sms_message;

    close(MAIL);

    
}

sub give_error{

    my ($self, $error_code) = @_;

    my $error_msg = $self->{'cfg'}->val('sms_error_codes',$error_code);
    if($error_msg) {
	print $error_msg,"\n" if($debug);
    }
    else{
	print "error_code: $error_code not yet defined\n";
    }
}

1;
__END__

=head1 AUTHOR

Klaus Tillmann, nico@mcs-cityline.net

=head1 SEE ALSO

perl(1).

=cut

