package MCS::CGI::SendToFriend;

#$Id: SendToFriend.pm,v 1.12 2003/12/15 14:20:25 nico Exp $
#$Log: SendToFriend.pm,v $
#Revision 1.12  2003/12/15 14:20:25  nico
#[BUG] Ups, Regex war ja noch gar nicht fertig.
#
#Revision 1.11  2003/12/15 14:16:44  nico
#[UPD] SpamSchutz integiert
#      es kann nur noch eine eMail Adresse je recip_email angegeben werden
#
#Revision 1.9  2003/03/24 12:06:09  nico
#[UPD] HTML::Template  durch  MCS::CGI::HTML::Template
#      ersetzt
#
#Revision 1.8  2002/02/25 14:29:54  nico
#* [FIX] HTML-Mail
#        - die Variablen Erstetzung fuer FROM hatte nicht funktioniert
#	- falls eine zu ersetzende Varialbe nicht definiert ist
#          wird jetzt ein Leersting "" an ihrer Stelle ausgegeben
#
#Revision 1.7  2002/02/22 19:19:35  nico
#* [NEW] HTML-Mail ist nun eine weitere Option
#        dafuer muss in der Konfigurationsdatei ungefair folgendes angeben sein:
#
#	[sendtofriend_message]
#        ...
#	mail_template = /send-to-a-friend/message.tmpl.html
#        mail_mimetype = text/html
#
#	Dabei werden folgende Ersetzungen in dem Template vorgenommen:
#
#	FROM_NAME - Name des Absenders
#	FROM	  - Email des Absenders
#	TO_NAME   - Name des Empfaengers
#	TO	  - Email des Empfaengers
#	CC	  - Email der CC Empfaenger laut Konfigurationsdatei
#	MESSAGE   - Mitteilung des Empfaengers
#

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use CGI::Carp;
use MCS::CGI::Conf;
use MCS::CGI::Standard;
use MCS::CGI::HTML::Template;
use IO::File;
use Text::ParseWords;
use CGI qw(:standard escapeHTML);
use MIME::Base64;
use MIME::Lite;

use POSIX;

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader	
	MCS::CGI::Standard
);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
  sendtofriend
  sendtofriend_admin
);


$VERSION = sprintf "%d.%02d", (q$Revision: 1.12 $ =~ /\d+/g);


=head1 NAME

MCS::CGI::SendToFriend - Standard-SendToFriend-Tool

=head1 VERSION

     $Id: SendToFriend.pm,v 1.12 2003/12/15 14:20:25 nico Exp $


=head1 SYNOPSIS


Hier ist das gesamte CGI-Skript (fE<uuml>nf Zeilen):

  #! /usr/bin/perl -wT
  use strict;
  $ENV{PATH} = '/bin:/usr/bin'; # Fuer Taint-Checking
  use MCS::CGI::SendToFriend qw( sendtofriend );
  my $cgi = new CGI;
  sendtofriend( $cgi);


=head1 DESCRIPTION

MCS::CGI::SendToFriend Administrationsschnittstelle.
 

=head1 CONFIG-FILE


	[sendtofriend_path]
	stored_entry 	= /sendtofriend/hidden/sendtofriend.txt

	[sendtofriend_template]
	danke			= /sendtofriend/danke.html

	[sendtofriend_input]
	fields	= lastname, firstname, street, city, zipcode, country, email

	[sendtofriend_text]
	date_format	= %Y.%m.%d-%H:%M:%S


	[admin]
	name    = mcs
	pass    = cityline

=head1 CONSTANTS

=over

=item *  EXCLUSIVE_LOCK 2

=item *  UNLOCK  8

=back 

=cut 

sub EXCLUSIVE_LOCK { return 2;}
sub UNLOCK { return 8;}


=head1 FUNCTIONS
# Preloaded methods go here.

=over

=item * sendtofriend

 sendtofriend($cgi) ist die Modulschnittstellen nach aussen


=cut

sub sendtofriend{
    my ($cgi )  = @_;
    unless ( $ENV{PATH_TRANSLATED} ) {
	$ENV{PATH_TRANSLATED} = "$ENV{DOCUMENT_ROOT}/sendtofriend/sendtofriend.conf";
    }
    
    my $cfg = new MCS::CGI::Conf;
    store_new_entry($cfg,$cgi);
}


=item * store_new_entry ( cfg, $cgi); 

speichert den neuen Eintrag nach ueberpruefung. 
    
=cut


sub store_new_entry {
    my ($cfg, $cgi) = @_;
    my $i = 0;
    my $j = 0;
    my $tmp;
    
    
    my ($sender_name, $sender_email, $recip_name, @recip_name, $recip_email, @recip_email, @recip_name_param, @recip_email_param, $message);
    
    $sender_name = $cfg->val('sendtofriend_input', 'sender_name');	
    $sender_name = param($sender_name);
    
    $sender_email = $cfg->val('sendtofriend_input', 'sender_email');	
    $sender_email = param($sender_email);
    
    $message = $cfg->val('sendtofriend_input', 'message');	
    $message = param($message);
    
    $recip_name = $cfg->val('sendtofriend_input','recip_name');
    $recip_name =~ s/[^a-z-_A-Z0-9]/ /g;
    @recip_name = split /\s+/,$recip_name;
    
    $recip_email = $cfg->val('sendtofriend_input','recip_email');
    $recip_email =~ s/[^a-z-_A-Z0-9]/ /g;
    @recip_email = split /\s+/,$recip_email;
    
    
    for ($j=0 ; $j <= $#recip_name ; $j++) {
		$recip_name_param[$j] = param($recip_name[$j]);
		if( is_valid_email( param($recip_email[$j] )) ){
			$recip_email_param[$j] = param($recip_email[$j]);
		}
		else{
			#minimaler Spamschutz: 
			$recip_email_param[$j] =~ s/^(.*?)[\s;,].*/$1/;
		}
    }
    
    
    
#### Sendmail-Stuff
    my $greeting = $cfg->val('sendtofriend_message', 'greeting');
    my $download_text = $cfg->val('sendtofriend_message', 'download_text');
    my $download_link = $cfg->val('sendtofriend_message', 'download_link');
    my $homepage_text = $cfg->val('sendtofriend_message', 'homepage_text');
    my $homepage_link = $cfg->val('sendtofriend_message', 'homepage_link');
    
# Schleife ueber die Anzahl der in der Konfig enthaltenen
# Empfaenger-Felder	
    for ($j=0 ; $j <= $#recip_name ; $j++) {
        if ($recip_name_param[$j] ne "") {
            if( $cfg->val('sendtofriend_message','mail_template')) {
                send_html_mail_by_template(  $cfg, { from         => $sender_email,
                                                     from_name  => $sender_name,
                                                     to           => $recip_email_param[$j],
                                                     to_name  => $recip_name_param[$j],
                                                     message      => $message});
            }
            elsif( $cfg->val('sendtofriend_attach','attach_file')){
                send_html_mail_with_attachment( $cfg, { from         => $sender_email,
                                        from_name  => $sender_name,
                                        to           => $recip_email_param[$j],
                                        to_name  => $recip_name_param[$j],
                                        greeting => $greeting,
                                        message      => $message,
                                        download_text => $download_text,
                                        download_link => $download_link,
                                        download_text => $download_text,
                                        download_link => $download_link,
                                        homepage_text => $homepage_text,
                                        homepage_link => $homepage_link
                                        });
            }
            else{
                                send_html_mail( $cfg, { from         => $sender_email,
                                        from_name  => $sender_name,
                                        to           => $recip_email_param[$j],
                                        to_name  => $recip_name_param[$j],
                                        greeting => $greeting,
                                        message      => $message,
                                        download_text => $download_text,
                                        download_link => $download_link,
                                        download_text => $download_text,
                                        download_link => $download_link,
                                        homepage_text => $homepage_text,
                                        homepage_link => $homepage_link
                                        });
            }
            
        }
    }
    # Schleifenende		
    
    #
    my $path_data = $ENV{DOCUMENT_ROOT}. $cfg->val('sendtofriend_path','stored_entry');
    
    if($path_data =~ /^([-_\w.\/]+)$/ ) {
	$path_data = $1;
	
	unless( open(DATA,">> $path_data") ) {
	    print STDERR __FILE__. " konnte $path_data nicht zum schreiben oeffnen!!!\n";
	    die "konnte Datei nicht zum schreiben oeffnen!!!\n";
	}
	#flock (DATA, $exclusive_lock);
	flock (DATA, EXCLUSIVE_LOCK);
	
	select(DATA);	
	
	print POSIX::strftime( $cfg->val('sendtofriend_text','date_format'), localtime(time));    	
	    
	print ";";
	    
	print $sender_name,";";
	print $sender_email,";";
	
	for ($j = 0; $j <= $#recip_name; $j++) {
	    if ($recip_email_param[$j] ne "") {
		print $recip_name_param[$j],";";
		print $recip_email_param[$j],";";
	    }
	}
	
	print "\n";
	
	flock (DATA, UNLOCK);
	
	close(DATA);
	
	select(STDOUT);	
	if( $cfg->val('sendtofriend_input','source') eq 'flash'){
	    print CGI::header(-type=> "text/plain");
	    print "staf_ok=1";
	}
	else{
	    my $redirect_url = $cfg->val('sendtofriend_template','danke');
	    if( $redirect_url) {
		print CGI::redirect(-uri => $redirect_url);
	    }
	    else{
		print CGI::header(-type=> "text/plain");
		print "unvorhergesehener Fehler\n Redirect URL dont defined\n";
		print STDERR __FILE__,__LINE__,"\nFehler weil \$cfg->val('sendtofriend_template','danke') nicht definiert ist\n";
	    }
	}
    }
    else {
	#path_data enthaelt ungueltige Zeichen
	if( $cfg->val('sendtofriend_input','source') eq 'flash'){
	    print CGI::header(-type=> "text/plain");
	    print "staf_ok=0";
	}
	else{
	    print CGI::header(-type=> "text/plain");
	    print "unvorhergesehener Fehler\n";
	}
    }
    return 1;
}


sub sendtofriend_admin{
    my ($cgi )  = @_;
    unless ( $ENV{PATH_TRANSLATED} ) {
	$ENV{PATH_TRANSLATED} = "$ENV{DOCUMENT_ROOT}/sendtofriend/sendtofriend.conf";
    }
	my $p = new MCS::CGI::HTML::Template;	
    my $cfg = new MCS::CGI::Conf;
	my @env = (); 		#Array fuer die Templatevar.

    unless (is_valid_admin($cfg,$cgi)) {
	die_error($cfg, $cfg->val("sendtofriend_text", "admin_only") );
    }
	my $path_data = $ENV{DOCUMENT_ROOT}. $cfg->val('sendtofriend_path','stored_entry');
	my $path_tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("sendtofriend_template","admin");

	if($path_data =~ /^([-_\w.\/]+)$/ ) {
           $path_data = $1;

	unless( open(DATA,"< $path_data") ) {
		print STDERR __FILE__. " konnte $path_data nicht zum schreiben oeffnen!!!\n";
		die "konnte Datei nicht zum schreiben oeffnen!!!\n";
	}

	flock (DATA, EXCLUSIVE_LOCK);

	select(DATA);	
	my $count = 0;
	my ($line,$i,$entry_number);

	while (defined ($line = <DATA>)) {
		chomp $line;
	#erste Schleife die alle Zeilen aus der Datei verarbeitet
		if ($line ne "") {
			my @entry = parse_csv($line);

		$entry_number++;
		}
	}
	flock(DATA, UNLOCK);
	close(DATA);
	select(STDOUT);
		
	print "Content-type: text/html","\n\n";

	my $tm = POSIX::strftime( $cfg->val('sendtofriend_text','date_format_output'), localtime(time));    	
	$p->var(TIME => $tm);


	my $fh = new IO::File $path_tmpl
	or die "Cannot open '$path_tmpl': $!";
	
	$p->var(VOTER => $entry_number);

	$p->parse_file($fh);

	}
}





sub die_error {
    my ($cfg, $public_error_msg, $public_long_error_msg ) =@_;
    my $parse = new MCS::CGI::HTML::Template;
    my $tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("sendtofriend_template","error");
    print "Content-type: text/html\n\n";
    $parse->var(ERROR => $public_error_msg);
    $parse->var(ERROR_LANG_TEXT => $public_long_error_msg || "");
    $parse->parse_file($tmpl);
    die;
}


sub is_valid_admin{
    my ($cfg,$cgi) = @_;
    return ( ($cfg->val('admin','name') eq $cgi->param('admin_name') ) &&
	     ($cfg->val('admin','pass') eq $cgi->param('admin_pass') ) );
    
}

sub parse_csv {
	return &quotewords(";",0,$_[0]);
}





sub send_html_mail {

    my ($cfg, $hash_ref) = @_;
    my $subject = $cfg->val('sendtofriend_message','subject') || "Screensaver";
    my $sendmail = $cfg->val('mail', 'sendmail');
    die
	"$sendmail insecure (must be an absolute path without whitespace)\n"
	    unless $sendmail =~ m|^/[^\s]+$|;
    die "$sendmail not found or not executable.\n"
	unless -x $sendmail;

	open SENDMAIL, "| $sendmail -t -oi"
        or die "Cannot open sendmail: $!\n";

    print SENDMAIL "From: $$hash_ref{'from'}\n";	
    print SENDMAIL "To: $$hash_ref{'to'}\n";
    print SENDMAIL "Subject: $subject\n";
    
    print SENDMAIL <<EOF;
Content-Type: text/html; charset=us-ascii
Content-Transfer-Encoding: 7bit\n\n
$$hash_ref{'greeting'} $$hash_ref{'from_name'}<br>
message:<br>
$$hash_ref{'message'} <br>
$$hash_ref{'download_text'} <a href="$$hash_ref{'download_link'}">$$hash_ref{'download_link'}</a> <br>
$$hash_ref{'homepage_text'} <a href="$$hash_ref{'homepage_link'}">$$hash_ref{'homepage_link'}</a> \n
EOF

    close (SENDMAIL);
}



sub send_html_mail_with_attachment {

    my ($cfg, $hash_ref) = @_;

    my ($boundary, $subject,$attach_file, $attach_mime, $msg_text);
    $subject = $cfg->val('sendtofriend_message','subject') || "Screensaver";
    
    my $msg = 
      MIME::Lite->new(
		      From    => $$hash_ref{'from'},
		      To      => $$hash_ref{'to'},
		      Cc      => $$hash_ref{'cc'},
		      Bcc     => $$hash_ref{'bcc'},
		      Subject => $subject,
		      Type    =>'multipart/mixed'
		      );


    $msg_text =  <<EOF;
Hi $$hash_ref{'to_name'},
$$hash_ref{'greeting'} $$hash_ref{'from_name'}<br>
message:<br>
$$hash_ref{'message'} <br>
$$hash_ref{'download_text'} <a href="$$hash_ref{'download_link'}">$$hash_ref{'download_link'}</a> <br>
$$hash_ref{'homepage_text'} <a href="$$hash_ref{'homepage_link'}">$$hash_ref{'homepage_link'}</a>
EOF


    $msg->attach( Type     =>'TEXT',
		  Data     => $msg_text
		  );


    my $sendmail = $cfg->val('mail', 'sendmail');
    die
	"$sendmail insecure (must be an absolute path without whitespace)\n"
	    unless $sendmail =~ m|^/[^\s]+$|;
    die "$sendmail not found or not executable.\n"
	unless -x $sendmail;

	open SENDMAIL, "| $sendmail -t "
        or die "Cannot open sendmail: $!\n";

    $attach_mime = $cfg->val('sendtofriend_attach', 'attach_mime');
    $attach_file = $cfg->val('sendtofriend_attach', 'attach_file');

    $msg->attach(Type     => $attach_mime,
		 Path     => "$ENV{DOCUMENT_ROOT}$attach_file",
		 Filename => $attach_file,
		 Disposition => 'attachment'
		 );
    $msg->print(\*SENDMAIL);
    close (SENDMAIL);

}


=item * send_html_mail_by_template 

generiert eine gestaltete E-Mail anhand des Temlates 
[sendtofriend_text]
mail_template = muster.tmpl.html


=cut 

sub send_html_mail_by_template {

    my ($cfg, $hash_ref) = @_;

    my ($boundary, $subject, $msg_text);
    my $parse = new MCS::CGI::HTML::Template;
    
    $subject = $cfg->val('sendtofriend_message','subject') || "Screensaver";
    my $mail_type =    $cfg->val('sendtofriend_message','mail_mimetype');
     $$hash_ref{'message'} =~ s/\n/<br>\n/g;

    $parse->var(TO => $$hash_ref{'to'}|| "");
    $parse->var(TO_NAME => $$hash_ref{'to_name'} ||"" );
    $parse->var(FROM => $$hash_ref{'from'} || "");
    $parse->var(FROM_NAME => $$hash_ref{'from_name'});
    $parse->var(CC => $$hash_ref{'cc'} || "");
    $parse->var(MESSAGE => $$hash_ref{'message'});
		
    my $sendmail = $cfg->val('mail', 'sendmail');
    die
	"$sendmail insecure (must be an absolute path without whitespace)\n"
	    unless $sendmail =~ m|^/[^\s]+$|;
    die "$sendmail not found or not executable.\n"
	unless -x $sendmail;

	open SENDMAIL, "| $sendmail -t "
        or die "Cannot open sendmail: $!\n";

    # $msg->print(\*SENDMAIL);
    print SENDMAIL <<EOF;
From: $$hash_ref{'from'}
To:   $$hash_ref{'to'}
Cc:   $$hash_ref{'cc'}
Subject: $subject
Content-Type: $mail_type; charset=us-ascii

EOF
    my $stdout = select(SENDMAIL);

    $parse->parse_file( $ENV{DOCUMENT_ROOT}.$cfg->val('sendtofriend_message','mail_template'));

    select($stdout);
    close (SENDMAIL);

}



# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__


# Below is the stub of documentation for your module. You better edit it!
=back

=head1 AUTHOR

Developer team <devel@mcs-cityline.net>
Patrick Surke <psurke@mcs-cityline.net>

=head1 SEE ALSO

perl(1).

=cut

