package MCS::CGI::Standard;

##
## MCS::CGI::Standard -- Standardfunktionen fuer CGI-Skripte
##
## $Id: Standard.pm,v 1.10 2005/09/06 14:11:37 nico Exp $
## Copyright (C) 2000 MCS Moorbek Computer Systeme GmbH
## Author: Klaus Tillmann <nico@cityline.net>
##


use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);

@EXPORT_OK = qw(
  uml2html
);

$VERSION = sprintf "%d.%02d", (q$Revision: 1.10 $ =~ /\d+/g);


=head1 NAME

MCS::CGI::Standard - Standardfunktionen fuer CGI-Skripte

=head1 SYNOPSIS

  use MCS::CGI::Standard;


=head1 DESCRIPTION

hier werden verschiedene Standard-Funktionen zur Verfuegung gestellt.
 

=cut 

=pod
 
An B<is_valid_email> wird ein String uebergeben, der daraufhin ueberprueft wird, ob es sich dabei um eine gueltige E-Mail Adresse handelt.

Rueckgabewert ist true oder falsch.

Achtung, wenn 4-stellige Toplevel-Domains eingefuert werden, muss diese Funktion angepasst werden.
 

 
=cut

sub is_valid_email{
     my ($email) = @_;
     unless ($email) { return undef;}

     if( give_valid_email($email) ){
         return 1;
     }
     else{
         return 0;
     }

}
=pod
 
An B<give_valid_email> wird ein String uebergeben, der daraufhin ueberprueft wird, ob es sich dabei um eine gueltige E-Mail Adresse handelt.

Rueckgabewert ist $email oder falsch.

Achtung, wenn 4-stellige Toplevel-Domains eingefuert werden, muss diese Funktion angepasst werden.
 

 
=cut

sub give_valid_email{
     my ($email) = @_;
     unless ($email) { return undef;}

     if( $email =~    /(^[a-zA-Z0-9._-]+\@[a-zA-Z][a-zA-Z0-9.-]+?\.[a-zA-Z]{2,6}$)/){
         return $1;
     }
     elsif ($email =~ /(<[a-zA-Z0-9._-]+\@[a-zA-Z][a-zA-Z0-9.-]+?\.[a-zA-Z]{2,6}>)$/ )  {
         return $1;
     }
     else{
         return 0;
     }

}


=pod 

B<uml2html> nimmt einen String und wandelt die 
Umlaute HTML-Codiert um. Leider gibt es Ueberschneidungen 
zwischen IBM PC und Mac.
Es wird angenommen, dass IBM PC haeufiger sind und deshalb
die IBM PC Variante bevorzugt. Es sei denn, es ist vorher
schon mal ein Mac Umlaut aufgetaucht. Dann wird auch \232
zum Macintosh "oe".


 

=cut

sub uml2html {
  my ($str)=@_;
  my $is_mac;

  # Leider gibt es Ueberschneidungen zwischen IBM PC und Mac.
  # Es wird angenommen, dass IBM PC haeufiger sind und deshalb
  # die IBM PC Variante bevorzugt. Es sei denn, es ist vorher
  # schon mal ein Mac Umlaut aufgetaucht. Dann wird auch \232
  # zum Macintosh "oe".
  $is_mac = 0;
  
  # Macintosh Umlaute nach HTML
  if ($str =~ s/\200/&Auml;/g) {$is_mac = 1};
  if ($str =~ s/\205/&Ouml;/g) {$is_mac = 1};
  if ($str =~ s/\206/&Uuml;/g) {$is_mac = 1};
  if ($str =~ s/\212/&auml;/g) {$is_mac = 1};
  if ($str =~ s/\237/&uuml;/g) {$is_mac = 1};
  if ($str =~ s/\247/&szlig;/g) {$is_mac = 1};
  if ($is_mac == 1) {$str =~ s/\232/&ouml;/g }
  
  # Latin1 Umlaute nach HTML
  $str =~ s/\304/&Auml;/g;
  $str =~ s/\326/&Ouml;/g;
  $str =~ s/\334/&Uuml;/g;
  $str =~ s/\344/&auml;/g;
  $str =~ s/\366/&ouml;/g;
  $str =~ s/\374/&uuml;/g;
  $str =~ s/\337/&szlig;/g;
  
  # IBM PC Umlaute nach HTML
  $str =~ s/\216/&Auml;/g;
  $str =~ s/\231/&Ouml;/g;
  $str =~ s/\232/&Uuml;/g;
  $str =~ s/\204/&auml;/g;
  $str =~ s/\224/&ouml;/g;
  $str =~ s/\201/&uuml;/g;
  $str =~ s/\341/&szlig;/g;
  
  $str =~ s/\264/\'/g;
  $str =~ s/\270/\`/g;
  $str =~ s/\n/<br>/g; # Zeilenumbruch setzten.
  return $str;
}

sub install {
  print <<EOF;

  Funktion ist nicht implementiert !

Diese Funktionalitt sollte von der jeweiligen Sub-Class zur Verfuegung gestellt werden
EOF
}



# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!


=head1 AUTHOR

Klaus Tillmann <nico@mcs-cityline.net>

=head1 SEE ALSO

perl(1).

=cut
