package MCS::CGI::Voting;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use CGI::Carp;
use MCS::CGI::Conf;
use MCS::CGI::HTML::Template;
use IO::File;
use Text::ParseWords;
use CGI qw(:standard escapeHTML);

use POSIX;

require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader	
	MCS::CGI::Standard
);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
  voting
  voting_admin
  voting_admin_output
);

#@EXPORT_OK = qw(

#);


$VERSION = sprintf "%d.%02d", (q$Revision: 1.5 $ =~ /\d+/g);


=head1 NAME

MCS::CGI::Voting - Standard-Voting-Tool

=head1 VERSION

     $Id: Voting.pm,v 1.5 2003/03/24 12:00:55 nico Exp $


=head1 SYNOPSIS


Hier ist das gesamte CGI-Skript (fE<uuml>nf Zeilen):

  #! /usr/bin/perl -wT
  use strict;
  $ENV{PATH} = '/bin:/usr/bin'; # Fuer Taint-Checking
  use MCS::CGI::Voting qw( voting );
  my $cgi = new CGI;
  voting( $cgi);

Hier ist das gesamte CGI-Skript (fE<uuml>nf Zeilen) fE<uuml>r die Administration:

  #! /usr/bin/perl -wT
  use strict;
  $ENV{PATH} = '/bin:/usr/bin'; # Fuer Taint-Checking
  use MCS::CGI::Voting qw( voting_admin );
  my $cgi = new CGI;
  voting_admin($cgi);


=head1 DESCRIPTION

MCS::CGI::Voting Administrationsschnittstelle.
 

=head1 CONFIG-FILE


	[voting_path]
	stored_entry 	= /voting/hidden/voting.txt

	[voting_template]
	fehler_voting	= /voting/fehler_voting.html
	fehler_name		= /voting/fehler_name.html
	danke			= /voting/danke.html

	[voting_input]
	voting_fields	= herzbrennt, links, sonne,ichwill, feuer, mutter, spieluhr, zwitter, reinraus, adios, nebel
	address_fields	= lastname, firstname, street, city, zipcode, country, email

	[voting_text]
	date_format	= %Y.%m.%d-%H:%M:%S


	[admin]
	name    = mcs
	pass    = cityline

=head1 CONSTANTS

=over

=item *  EXCLUSIVE_LOCK 2

=item *  UNLOCK  8

=back 

=cut 

sub EXCLUSIVE_LOCK { return 2;}
sub UNLOCK { return 8;}


=head1 FUNCTIONS
# Preloaded methods go here.

=over

=item * voting

 voting($cgi) ist die Modulschnittstellen nach aussen


=cut

sub voting{
    my ($cgi )  = @_;
    unless ( $ENV{PATH_TRANSLATED} ) {
	$ENV{PATH_TRANSLATED} = "$ENV{DOCUMENT_ROOT}/voting/voting.conf";
    }
	
    my $cfg = new MCS::CGI::Conf;

    store_new_entry($cfg,$cgi) 
}


=item * store_new_entry ( cfg, $cgi); 

speichert den neuen Eintrag nach ueberpruefung. 

=cut

sub check_if_every_element_defined {
	my ($array_ref) = @_;
	my @array = @$array_ref;
	for (my $i = 0; $i <= $#array ; $i++) {
		if ( !$array[$i] ) {
		return 0;
		}
	}
	return 1;
}


sub store_new_entry {
    my ($cfg, $cgi) = @_;
	my $i = 0;
	my $j = 0;
	my $tmp;

	my $path_data = $ENV{DOCUMENT_ROOT}. $cfg->val('voting_path','stored_entry');
#	"/www/patrick5.mcs-hh.de/htdocs/voting.data";



	my ($fields, @fields, @fields_param, @field, $voter, @voter, @voter_param);

	$fields = $cfg->val('voting_input','voting_fields');
	$fields =~ s/[^a-z-_A-Z0-9]/ /g;
	@fields = split /\s+/,$fields;

    for ($i=0 ; $i <= $#fields ; $i++) {
	   $fields_param[$i] = param($fields[$i]);
    }

	$voter = $cfg->val('voting_input','address_fields');
	$voter =~ s/[^a-z-_A-Z0-9]/ /g;
	@voter = split /\s+/,$voter;

    for ($j=0 ; $j <= $#voter ; $j++) {
	   $voter_param[$j] = param($voter[$j]);
    }

	if ( check_if_every_element_defined(\@fields_param) == 0 ) {
		print CGI::redirect(-uri => $cfg->val('voting_template','fehler_voting'));
	}

	elsif ( check_if_every_element_defined(\@voter_param) == 0 ) {
		print CGI::redirect(-uri => $cfg->val('voting_template','fehler_name'));
	}

	else {
	    if($path_data =~ /^([-_\w.\/]+)$/ ) {
               $path_data = $1;

		unless( open(DATA,">> $path_data") ) {
			print STDERR __FILE__. " konnte $path_data nicht zum schreiben oeffnen!!!\n";
			die "konnte Datei nicht zum schreiben oeffnen!!!\n";
		}
			#flock (DATA, $exclusive_lock);
		flock (DATA, EXCLUSIVE_LOCK);

		select(DATA);	

		for ($i = 0; $i <= $#fields_param; $i++) {
			print $fields_param[$i],";";
		}

		for ($j = 0; $j <= $#voter_param; $j++) {
			print $voter_param[$j],";";
		}

		print $ENV{"REMOTE_ADDR"},";";
		
		#printf ("%04d-%02d-%02d\n",$tm->year+1900,($tm->mon)+1,$tm->mday);
		
		print POSIX::strftime( $cfg->val('voting_text','date_format'), localtime(time));    	

		print "\n";

		flock (DATA, UNLOCK);

		close(DATA);

		select(STDOUT);	

		print CGI::redirect(-uri => $cfg->val('voting_template','danke'));
             }
	     else {
		#path_data enthaelt ungueltige Zeichen
		print CGI::header(-type=> "text/plain");
		print "unvorhergesehener Fehler\n";
	      }
	}

    return 1;
    
}

sub die_error {
    my ($cfg, $public_error_msg, $public_long_error_msg ) =@_;
    my $parse = new MCS::CGI::HTML::Template;
    my $tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("voting_template","error");
    print "Content-type: text/html\n\n";
    $parse->var(ERROR => $public_error_msg);
    $parse->var(ERROR_LANG_TEXT => $public_long_error_msg || "");
    $parse->parse_file($tmpl);
    die;
}

sub is_valid_admin{
    my ($cfg,$cgi) = @_;
    return ( ($cfg->val('admin','name') eq $cgi->param('admin_name') ) &&
	     ($cfg->val('admin','pass') eq $cgi->param('admin_pass') ) );
    
}

sub voting_admin{
    my ($cgi )  = @_;
    unless ( $ENV{PATH_TRANSLATED} ) {
	$ENV{PATH_TRANSLATED} = "$ENV{DOCUMENT_ROOT}/voting/voting.conf";
    }
	my $p = new MCS::CGI::HTML::Template;	
    my $cfg = new MCS::CGI::Conf;
	my @env = (); 		#Array fuer die Templatevar.

	my ($voter, @voter, @voter_param);

    unless (is_valid_admin($cfg,$cgi)) {
	die_error($cfg, $cfg->val("voting_text", "admin_only") );
    }
	my $path_tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("voting_template","admin");
	my $fh = new IO::File $path_tmpl
	or die "Cannot open '$path_tmpl': $!";

	$voter = $cfg->val('voting_input','address_fields');
	$voter =~ s/[^a-z-_A-Z0-9]/ /g;
	@voter = split /\s+/,$voter;

    for (my $i=0 ; $i <= $#voter ; $i++) {
		push @env, { VOTER => $voter[$i]};
		$p->var(ENV => \@env);	
	}


	print "Content-type: text/html","\n\n";

	$p->parse_file($fh);
}

sub parse_csv {
	return &quotewords(";",0,$_[0]);
}

sub check_if_double_voting{
	my ($entry_number,$fields_ref,$entry_ref) = @_;
	my @fields = @$fields_ref;
	my @entry = @$entry_ref;
	my %voting_data = ();
	keys(%voting_data) = 5;			
	$voting_data{$entry_number} = [ @entry ];
}

sub voting_admin_output{
#    my ($cgi,$votingtype,$entrycheck )  = @_;
    my ($cgi)  = @_;
	my $votingtype = $cgi->param('votingtype');
	my $entrycheck = $cgi->param('entrycheck');
    unless ( $ENV{PATH_TRANSLATED} ) {
		$ENV{PATH_TRANSLATED} = "$ENV{DOCUMENT_ROOT}/voting/voting.conf";
    }

#    for ($i=0 ; $i <= $#fields ; $i++) {
#	   $fields_param[$i] = param($fields[$i]);
#    }

	my $output;
	my $cfg = new MCS::CGI::Conf;
	my $path_data = $ENV{DOCUMENT_ROOT}. $cfg->val('voting_path','stored_entry');

	my ($fields, @fields, @fields_param, @field, $voter, @voter, @voter_param, $song_names, @song_names);

	$fields = $cfg->val('voting_input','voting_fields');
	$fields =~ s/[^a-z-_A-Z0-9]/ /g;
	@fields = split /\s+/,$fields;

	$song_names = $cfg->val('voting_input','song_names');
	$song_names =~ s/[^a-z-_A-Z0-9]/ /g;
	@song_names = split /\s+/,$song_names;


	my $p = new MCS::CGI::HTML::Template;	
	
#    unless (is_valid_admin($cfg,$cgi)) {
#	die_error($cfg, $cfg->val("voting_text", "admin_only") );
#    }

	my $path_tmpl;
	if ($votingtype eq 'each'){
		$path_tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("voting_template","admin-output-each");
	}
	if ($votingtype eq 'position'){
		$path_tmpl = $ENV{DOCUMENT_ROOT}. $cfg->val("voting_template","admin-output-position");
	}
	
	if($path_data =~ /^([-_\w.\/]+)$/ ) {
           $path_data = $1;

	unless( open(DATA,"< $path_data") ) {
		print STDERR __FILE__. " konnte $path_data nicht zum schreiben oeffnen!!!\n";
		die "konnte Datei nicht zum schreiben oeffnen!!!\n";
	}

	flock (DATA, EXCLUSIVE_LOCK);

	select(DATA);	
	my $count = 0;
	my $count_ber = 0;
	my @title = ();		#Array fuer Daten der einzelnen Titel- bzw. Voting-Daten
	my @title_ber = ();
	my @position = ([],[]);
	my @env = (); 		#Array fuer die Templatevar.
	my @env1 = (); 		#Array fuer die Templatevar.
	my @env2 = (); 		#Array fuer die Templatevar.
	my @env3 = (); 		#Array fuer die Templatevar.
	my @env3 = (); 		#Array fuer die Templatevar.
	my @env_ber = ();
#	my $ranking = 0;
#	my $rounded_ranking = 0;
	my @ranking = ();
	my @multi_env = ([],[]);
	my $entry_number = 0;
	my $entry_number_ber = 0;
	my ($line,$vote,$vote_ber,$rounded_vote,$rounded_vote_ber,$i);
# Hier werden die Daten aus dem Daten-Dokument verarbeitet und in 
# zwei Arrays geschrieben
	while (defined ($line = <DATA>)) {
		chomp $line;
	#erste Schleife die alle Zeilen aus der Datei verarbeitet
		if ($line ne "") {
			my @entry = parse_csv($line);

			for ($i = 0; $i < scalar(@fields); $i++) {         #Iteration ber Titel bzw. Platz 
				my $j = $entry[$i];
				$position[$i][$j]++;
				$title[$i] = $title[$i] + $entry[$i];		   #	
				if (check_if_double_voting ($entry_number, \@fields, \@entry) != 1) {
					$title_ber[$i] = $title_ber[$i] + $entry[$i];				
					$entry_number_ber++;
				}
			}
		$entry_number++;
		}
	}
# Berechnung beim Titel-voting

		if ($votingtype eq 'each'){
			for ($i = 0; $i < scalar(@fields); $i++) {
				$vote = $title[$i]/$entry_number;
				$rounded_vote = sprintf("%.2f", $vote);	
				if ($entry_number_ber != 0) {
					$vote_ber = $title_ber[$i]/$entry_number_ber;
				}
				else {
				$vote_ber = 0;
				}
				$rounded_vote_ber = sprintf("%.2f", $vote_ber);	
					$p->var(TYPE => "Titelvoting");

				push @env, { COUNT => $fields[$i], SUM => $title[$i], VAL => $rounded_vote };
					$p->var(ENV => \@env);
					$p->var(VOTER => $entry_number);

				push @env_ber, { COUNT => $fields[$i], SUM_BER => $title_ber[$i], VAL_BER => $rounded_vote_ber };
        			 $p->var(ENV_BER => \@env_ber);
					 $p->var(VOTER_BER => $entry_number_ber);
			}
		}

#Berechnung beim Platz-voting
		
		if ($votingtype eq 'position'){
			$p->var(TYPE => "Platzvoting");
#			for ($i = 0; $i < scalar(@fields); $i++) {
			for ($i = 0; $i <= (scalar(@fields)+1); $i++) {
		         sub position { return $fields[$i] eq $fields[$i]}
        		 $p->var("POSITION".$i => \&position);
		         sub ranking { return 1 eq 1}
        		 $p->var(RANKING => => \&ranking);

				for (my $j = 1; $j < 10; $j++) {
					my $ranking_tmp; 
					$vote = ($position[$i][$j]*100)/$entry_number;
					$rounded_vote = sprintf("%.2f", $vote);	

					for (my $k = 0; $k <= scalar(@fields); $k++) {
						my $tmp = ($position[$k][$j]*(scalar(@fields)-$k));
						$ranking_tmp = $ranking_tmp + $tmp;
					}
					my $ranking = $ranking_tmp;
					my $rounded_ranking = sprintf("%.2f", $ranking);	
					if ($entry_number_ber != 0) {
						$vote_ber = $title_ber[$i]/$entry_number_ber;
					}
					else {
						$vote_ber = 0;
					}
					$rounded_vote_ber = sprintf("%.2f", $vote_ber);	
	#Ausgabe des Platz-Voting	
	
					if ($i eq 0) {	
						push @env1, { TITEL => $song_names[$j-1], VAL => $rounded_vote};
						$p->var(ENV1 => \@env1);
						$p->var(POS1=> $fields[$i]); 
					}
					if ($i eq 1) {	
						push @env2, { TITEL => $song_names[$j-1], VAL => $rounded_vote};
						$p->var(ENV2 => \@env2);
						$p->var(POS2=> $fields[$i]); 
					}
					if ($i eq 2) {	
						push @env3, { TITEL => $song_names[$j-1], VAL => $rounded_vote};
						$p->var(ENV3 => \@env3);
						$p->var(POS3=> $fields[$i]); 
					}
					if ($i eq 3) {	
						push @ranking, { TITEL => $song_names[$j-1], VAL => $ranking};
						$p->var(RANK => \@ranking);
						$p->var(POS4=> $fields[$i]); 
					}
					$p->var(VOTER => $entry_number);
				}
			}
		}
	
	flock(DATA, UNLOCK);
	close(DATA);
	select(STDOUT);
		
	print "Content-type: text/html","\n\n";

	my $tm = POSIX::strftime( $cfg->val('voting_text','date_format_output'), localtime(time));    	
	$p->var(TIME => $tm);


	my $fh = new IO::File $path_tmpl
	or die "Cannot open '$path_tmpl': $!";

	$p->parse_file($fh);

	}
}



# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__


# Below is the stub of documentation for your module. You better edit it!
=back

=head1 AUTHOR

Patrick Surke <psurke@mcs-cityline.net>

=head1 SEE ALSO

perl(1).

=cut


